/*
 * hello-getopt.c --
 *
 *      This program uses the stdio library to print a short messages.
 *	The messages depend on the arguments provided to the program.
 *	The -g options can be used to set the greeting word. Note that
 *	we check whether buffered output stream managed to write the
 *	messages.
 */

#define _POSIX_C_SOURCE 2

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

static void hello(char *greet, char *thingy)
{
    if (printf("%s %s!\n", greet, thingy) < 0) {
        perror("printf");
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
    char *greet = "Hello";
    int opt;

    while ((opt = getopt(argc, argv, "g:")) != -1) {
	switch (opt) {
	case 'g':
	    greet = optarg;
	    break;
	default:
	    fprintf(stderr, "Usage: hello-getopt [-g greet] [names]\n");
	    exit(EXIT_FAILURE);
	}
    }
    
    if (argc == optind) {
        hello(greet, "world");
    } else {
        for (int i = optind; i < argc; i++) {
            hello(greet, argv[i]);
        }
    }

    if (fflush(stdout) == EOF) {
        perror("fflush");
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
